/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include "/build/libnvidia-container/src/libnvidia-container-1.19.0/src/nvc_rpc.h"
#pragma GCC diagnostic ignored "-Wmissing-prototypes"
#pragma GCC diagnostic ignored "-Wsign-conversion"
#pragma GCC diagnostic ignored "-Wunused-variable"

bool_t
xdr_ptr_t (XDR *xdrs, ptr_t *objp)
{
	register int32_t *buf;

	 if (!xdr_int64_t (xdrs, objp))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_init_res (XDR *xdrs, driver_init_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_init_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_shutdown_res (XDR *xdrs, driver_shutdown_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_shutdown_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_rm_version_res (XDR *xdrs, driver_get_rm_version_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_string (xdrs, &objp->driver_get_rm_version_res_u.vers, ~0))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_rm_version_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_cuda_version (XDR *xdrs, driver_cuda_version *objp)
{
	register int32_t *buf;

	 if (!xdr_u_int (xdrs, &objp->major))
		 return FALSE;
	 if (!xdr_u_int (xdrs, &objp->minor))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_cuda_version_res (XDR *xdrs, driver_get_cuda_version_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_driver_cuda_version (xdrs, &objp->driver_get_cuda_version_res_u.vers))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_cuda_version_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_device_arch (XDR *xdrs, driver_device_arch *objp)
{
	register int32_t *buf;

	 if (!xdr_u_int (xdrs, &objp->major))
		 return FALSE;
	 if (!xdr_u_int (xdrs, &objp->minor))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_arch_res (XDR *xdrs, driver_get_device_arch_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_driver_device_arch (xdrs, &objp->driver_get_device_arch_res_u.arch))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_arch_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_count_res (XDR *xdrs, driver_get_device_count_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_u_int (xdrs, &objp->driver_get_device_count_res_u.count))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_count_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_res (XDR *xdrs, driver_get_device_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_ptr_t (xdrs, &objp->driver_get_device_res_u.dev))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_minor_res (XDR *xdrs, driver_get_device_minor_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_u_int (xdrs, &objp->driver_get_device_minor_res_u.minor))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_minor_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_busid_res (XDR *xdrs, driver_get_device_busid_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_string (xdrs, &objp->driver_get_device_busid_res_u.busid, ~0))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_busid_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_uuid_res (XDR *xdrs, driver_get_device_uuid_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_string (xdrs, &objp->driver_get_device_uuid_res_u.uuid, ~0))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_uuid_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_model_res (XDR *xdrs, driver_get_device_model_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_string (xdrs, &objp->driver_get_device_model_res_u.model, ~0))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_model_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_brand_res (XDR *xdrs, driver_get_device_brand_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_string (xdrs, &objp->driver_get_device_brand_res_u.brand, ~0))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_brand_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_device_mig_mode (XDR *xdrs, driver_device_mig_mode *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->error))
		 return FALSE;
	 if (!xdr_u_int (xdrs, &objp->current))
		 return FALSE;
	 if (!xdr_u_int (xdrs, &objp->pending))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_mig_mode_res (XDR *xdrs, driver_get_device_mig_mode_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_driver_device_mig_mode (xdrs, &objp->driver_get_device_mig_mode_res_u.mode))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_mig_mode_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_max_mig_device_count_res (XDR *xdrs, driver_get_device_max_mig_device_count_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_u_int (xdrs, &objp->driver_get_device_max_mig_device_count_res_u.count))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_max_mig_device_count_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_mig_device_res (XDR *xdrs, driver_get_device_mig_device_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_ptr_t (xdrs, &objp->driver_get_device_mig_device_res_u.dev))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_mig_device_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_gpu_instance_id_res (XDR *xdrs, driver_get_device_gpu_instance_id_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_u_int (xdrs, &objp->driver_get_device_gpu_instance_id_res_u.id))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_gpu_instance_id_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_compute_instance_id_res (XDR *xdrs, driver_get_device_compute_instance_id_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_u_int (xdrs, &objp->driver_get_device_compute_instance_id_res_u.id))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->driver_get_device_compute_instance_id_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_driver_get_device_1_argument (XDR *xdrs, driver_get_device_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_u_int (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_minor_1_argument (XDR *xdrs, driver_get_device_minor_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_busid_1_argument (XDR *xdrs, driver_get_device_busid_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_uuid_1_argument (XDR *xdrs, driver_get_device_uuid_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_arch_1_argument (XDR *xdrs, driver_get_device_arch_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_model_1_argument (XDR *xdrs, driver_get_device_model_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_brand_1_argument (XDR *xdrs, driver_get_device_brand_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_mig_mode_1_argument (XDR *xdrs, driver_get_device_mig_mode_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_max_mig_device_count_1_argument (XDR *xdrs, driver_get_device_max_mig_device_count_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_mig_device_1_argument (XDR *xdrs, driver_get_device_mig_device_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	 if (!xdr_u_int (xdrs, &objp->arg3))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_gpu_instance_id_1_argument (XDR *xdrs, driver_get_device_gpu_instance_id_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_driver_get_device_compute_instance_id_1_argument (XDR *xdrs, driver_get_device_compute_instance_id_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_ptr_t (xdrs, &objp->arg2))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_nvcgo_init_res (XDR *xdrs, nvcgo_init_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		break;
	default:
		 if (!xdr_string (xdrs, &objp->nvcgo_init_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_nvcgo_shutdown_res (XDR *xdrs, nvcgo_shutdown_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		break;
	default:
		 if (!xdr_string (xdrs, &objp->nvcgo_shutdown_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_nvcgo_get_device_cgroup_version_res (XDR *xdrs, nvcgo_get_device_cgroup_version_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_u_int (xdrs, &objp->nvcgo_get_device_cgroup_version_res_u.vers))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->nvcgo_get_device_cgroup_version_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_nvcgo_find_device_cgroup_path_res (XDR *xdrs, nvcgo_find_device_cgroup_path_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		 if (!xdr_string (xdrs, &objp->nvcgo_find_device_cgroup_path_res_u.cgroup_path, ~0))
			 return FALSE;
		break;
	default:
		 if (!xdr_string (xdrs, &objp->nvcgo_find_device_cgroup_path_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_nvcgo_setup_device_cgroup_res (XDR *xdrs, nvcgo_setup_device_cgroup_res *objp)
{
	register int32_t *buf;

	 if (!xdr_int (xdrs, &objp->errcode))
		 return FALSE;
	switch (objp->errcode) {
	case 0:
		break;
	default:
		 if (!xdr_string (xdrs, &objp->nvcgo_setup_device_cgroup_res_u.errmsg, ~0))
			 return FALSE;
		break;
	}
	return TRUE;
}

bool_t
xdr_nvcgo_get_device_cgroup_version_1_argument (XDR *xdrs, nvcgo_get_device_cgroup_version_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->arg2, ~0))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->arg3))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_nvcgo_find_device_cgroup_path_1_argument (XDR *xdrs, nvcgo_find_device_cgroup_path_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->arg2))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->arg3, ~0))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->arg4))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->arg5))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_nvcgo_setup_device_cgroup_1_argument (XDR *xdrs, nvcgo_setup_device_cgroup_1_argument *objp)
{
	 if (!xdr_ptr_t (xdrs, &objp->arg1))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->arg2))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->arg3, ~0))
		 return FALSE;
	 if (!xdr_u_long (xdrs, &objp->arg4))
		 return FALSE;
	return TRUE;
}
